<?php

namespace Drupal\entity_translations_helper;

use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\content_translation\ContentTranslationManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\EventSubscriber\MainContentViewSubscriber;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\CloseDialogCommand;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Ajax\BeforeCommand;
use Drupal\Core\Entity\ContentEntityFormInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Component\Utility\Html;
use Drupal\Core\Language\LanguageInterface;
use Drupal\Core\Link;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Render\Element;
use Drupal\Core\Field\EntityReferenceFieldItemListInterface;

/**
 * Entity translations helper "EntityTranslationsHelper" service object.
 *
 * @SuppressWarnings(PHPMD.CouplingBetweenObjects)
 */
class EntityTranslationsHelper {

  const SELECTOR_BASE = 'entity-translations-helper';
  const SELECTOR_MODAL = self::SELECTOR_BASE . '-modal';
  const SELECTOR_WRAPPER = self::SELECTOR_BASE . '-wrapper';
  const SELECTOR_FIELD_BASE = self::SELECTOR_BASE . '-field-';
  const SELECTOR_ENTITY_BASE = self::SELECTOR_BASE . '-entity-';

  /**
   * A request stack object.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The content translation manager.
   *
   * @var \Drupal\content_translation\ContentTranslationManagerInterface
   */
  protected $contentTranslationsManager;

  /**
   * Entity type manager which performs the upcasting in the end.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a "EntityTranslationsHelper" object.
   *
   * @param \Symfony\Component\HttpFoundation\RequestStack $request_stack
   *   A request stack object.
   * @param \Drupal\content_translation\ContentTranslationManagerInterface $content_translations_manager
   *   The content translation manager service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(RequestStack $request_stack, ContentTranslationManagerInterface $content_translations_manager, EntityTypeManagerInterface $entity_type_manager) {
    $this->requestStack = $request_stack;
    $this->contentTranslationsManager = $content_translations_manager;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * Alter main form.
   *
   * @param array $form
   *   Form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form state.
   */
  public function alterMainForm(array &$form, FormStateInterface $form_state): void {
    $entity = $this->alterMainFormGetValidEntity($form_state);
    if ($entity instanceof ContentEntityInterface) {
      self::alterMainFormEntityBuidlFieldDetails($form, $entity);
      self::alterMainFormBuidlWrapperDetails($form);
    }
  }

  /**
   * Alter modal form.
   *
   * @param array $form
   *   Form.
   */
  public function alterModalForm(array &$form): void {
    if ($this->alterModalFormAjaxCheck()) {
      $form['#process'][] = [static::class, 'alterModalFormProcess'];
      $form['actions']['submit']['#ajax'] = [
        'callback' => [static::class, 'alterModalFormAjaxCallback'],
        'event' => 'click',
        'disable-refocus' => TRUE,
      ];
    }
  }

  /**
   * Alter modal form process.
   *
   * @param array $element
   *   Form element.
   *
   * @return array
   *   Form element processed.
   */
  public static function alterModalFormProcess(array $element): array {
    // Just leave main submit button to avoid any possible redirection.
    foreach (array_diff(Element::children($element['actions']), ['submit']) as $action_key) {
      $element['actions'][$action_key]['#access'] = FALSE;
    }
    return $element;
  }

  /**
   * Alter modal form ajax callback.
   *
   * @param array $form
   *   Form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form state.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   Ajax response.
   *
   * @SuppressWarnings(PHPMD.UnusedFormalParameter)
   */
  public static function alterModalFormAjaxCallback(array $form, FormStateInterface $form_state): AjaxResponse {
    $response = new AjaxResponse();
    $show_messages_inside_modal = TRUE;
    // Show any message & refresh link.
    if (!$form_state->hasAnyErrors()) {
      $response->addCommand(new CloseDialogCommand('.' . Html::cleanCssIdentifier(self::SELECTOR_MODAL)));
      $entity = $form_state->getFormObject()->getEntity();
      $link = self::entityBuildLinkForLanguage($entity, $entity->language());
      $link_html = \Drupal::service('renderer')->render($link);
      $response->addCommand(new ReplaceCommand('.' . Html::cleanCssIdentifier(self::SELECTOR_ENTITY_BASE . $entity->id()), $link_html));
      $show_messages_inside_modal = FALSE;
    }
    $response->addCommand(new BeforeCommand($show_messages_inside_modal ? '.' . Html::cleanCssIdentifier(self::SELECTOR_MODAL) . ' form' : 'form', [
      '#type' => 'status_messages',
    ]));
    return $response;
  }

  /**
   * Obtain main form valid entity.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form state.
   *
   * @return \Drupal\Core\Entity\ContentEntityInterface|null
   *   Entity when valid.
   */
  protected function alterMainFormGetValidEntity(FormStateInterface $form_state): ?ContentEntityInterface {
    $valid_entity = NULL;
    if ($form_state->getFormObject() instanceof ContentEntityFormInterface) {
      $entity = $form_state->getFormObject()->getEntity();
      $main_condition = $entity instanceof ContentEntityInterface && !$entity->isNew() && $entity->isTranslatable() && !$entity->isDefaultTranslation();
      if ($main_condition) {
        $settings = $this->contentTranslationsManager->getBundleTranslationSettings($entity->getEntityTypeId(), $entity->bundle());
        $valid_entity = !empty($settings['untranslatable_fields_hide']) ? $entity : $valid_entity;
      }
    }
    return $valid_entity;
  }

  /**
   * Modal form ajax check.
   *
   * @return bool
   *   When should proceed.
   */
  protected function alterModalFormAjaxCheck(): bool {
    $current_request = $this->requestStack->getCurrentRequest();
    $wrapper_format = $current_request->get(MainContentViewSubscriber::WRAPPER_FORMAT) ?? '';
    $is_ajax = str_contains($wrapper_format, 'drupal_ajax') || str_contains($wrapper_format, 'drupal_modal') || str_contains($wrapper_format, 'drupal_dialog');
    $has_ajax_query_param = $current_request->query->get('entity_translations_helper') === 'ajax';
    return $is_ajax && $has_ajax_query_param;
  }

  /**
   * Entity build link for language.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   Entity.
   * @param \Drupal\Core\Language\LanguageInterface $language
   *   Language.
   *
   * @return array
   *   Link ready to render.
   */
  protected static function entityBuildLinkForLanguage(ContentEntityInterface $entity, LanguageInterface $language): array {
    $link = [];
    if ($entity->isTranslatable()) {
      $url_options = [
        'language' => $language,
        'query' => ['entity_translations_helper' => 'ajax'],
        'attributes' => ['target' => '_blank'],
      ];
      // @see Related similar code, access add/edit or use translation form?
      if ($entity->hasTranslation($language->getId())) {
        $url = $entity->toUrl('edit-form', $url_options);
        // @code \Drupal\content_translation\Controller\ContentTranslationController::overview. @endcode
        // @code $url = $entity->toUrl('drupal:content-translation-edit', $url_options)->setRouteParameter('language', $language->getId()); @endcode
        $action_label = t('Edit');
        $label = $entity->getTranslation($language->getId())->label();
      }
      else {
        $url = $entity->toUrl('drupal:content-translation-add', $url_options)->setRouteParameter('source', $entity->getUntranslated()->language()->getId())->setRouteParameter('target', $language->getId());
        $action_label = t('Add');
        $label = $entity->label();
      }
      $link = Link::fromTextAndUrl(t(':action ":language" translation of ":label"', [
        ':language' => $language->getName(),
        ':action' => $action_label,
        ':label' => $label,
      ]), $url)->toRenderable();
      $link['#attributes']['class'][] = 'use-ajax';
      $link['#attributes']['class'][] = Html::cleanCssIdentifier(self::SELECTOR_ENTITY_BASE . $entity->id());
      $link['#attributes']['data-dialog-type'] = 'modal';
      $link['#attributes']['data-dialog-options'] = Json::encode([
        'height' => '75%',
        'width' => '75%',
        'autoResize' => TRUE,
        'classes' => ['ui-dialog-content' => Html::cleanCssIdentifier(self::SELECTOR_MODAL)],
      ]);
      $link['#attached']['library'][] = 'core/drupal.dialog.ajax';
      // Preview!? Condider it should also refreshed into ajax if added.
      // @code $preview = \Drupal::entityTypeManager()->getViewBuilder($entity->getEntityTypeId())->view($entity); @endcode
      // @code $link['#prefix'] = strip_tags(\Drupal::service('renderer')->render($preview), ['img']); @endcode
    }
    return $link;
  }

  /**
   * Alter main form build wrapper details.
   *
   * @param array $form
   *   Form.
   */
  protected static function alterMainFormBuidlWrapperDetails(array &$form): void {
    if (!empty($form['entity_translations_helper'])) {
      $form['entity_translations_helper'] += [
        '#type' => 'details',
        '#weight' => -999,
        '#title' => t('Manage related translations'),
        '#attributes' => [
          'class' => [Html::cleanCssIdentifier(self::SELECTOR_WRAPPER),
          ],
        ],
      ];
    }
  }

  /**
   * Alter main form entity build related field details.
   *
   * @param array $form
   *   Form.
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   Entity.
   */
  protected static function alterMainFormEntityBuidlFieldDetails(array &$form, ContentEntityInterface $entity): void {
    foreach ($entity->getFieldDefinitions() as $field_key => $field_definition) {
      $field_check = $field_definition->getType() == 'entity_reference' && !$field_definition->isTranslatable();
      $field = $field_check ? $entity->get($field_key) : NULL;
      if (!($field instanceof EntityReferenceFieldItemListInterface && !$field->isEmpty())) {
        continue;
      }
      $field_details = [];
      foreach ($field->referencedEntities() as $field_entity) {
        if (!($field_entity instanceof ContentEntityInterface && $field_entity->isTranslatable())) {
          continue;
        }
        $field_details[] = [
          '#type' => 'container',
          'link' => self::entityBuildLinkForLanguage($field_entity, $entity->language()),
        ];
      }
      if (!empty($field_details)) {
        $field_details += [
          '#type' => 'details',
          '#title' => $field_definition->getLabel(),
          '#attributes' => [
            'class' => [
              Html::cleanCssIdentifier(self::SELECTOR_FIELD_BASE . $field_key),
            ],
          ],
        ];
        $form['entity_translations_helper'][$field_key] = $field_details;
      }
    }
  }

}
